"use strict";

/* exported logger */

var logger = require('../logger');
var AbstractCustomMap = require('./AbstractCustomMap');
var CustomMapOptions = require('./CustomMapOptions');
var TouchMap = require('./TouchMap');

/**
 * Constructor of a submap. Submaps are created on application start. But dialog
 * (popup window) is initialized on demand using init function.
 *
 * @param customMap
 *          parent CustomMap
 * @param id
 *          identifier of the submap
 */
function Submap(customMap, model) {
    this.setCustomMap(customMap);

    AbstractCustomMap.call(this, model, new CustomMapOptions({
        element: customMap.getGoogleMap().getDiv(),
        markerOptimization: customMap.isMarkerOptimization(),
        bigLogo: customMap.isBigLogo(),
        customTouchInterface: customMap.isCustomTouchInterface(),
        project: null,
        debug: customMap.isDebug()
    }));

    this.initialized = false;
}

// implementation of object inheritance
Submap.prototype = Object.create(AbstractCustomMap.prototype);
Submap.prototype.constructor = Submap;

/**
 * This method initializes submap with gui component. Before this point submap
 * is created and contains data, but cannot be visualized in the browser.
 *
 * @param htmlTag
 *          html div tag where google map should be placed
 */
Submap.prototype.open = function (htmlTag) {
    var self = this;

    if (!this.initialized) {
        self.htmlTag = htmlTag;

        var mapOptions = self.createMapOptions(self.getLayouts().length);

        var contentDiv = document.createElement("div");
        contentDiv.setAttribute("name", "submap-div-" + self.getId());
        contentDiv.style.width = "100%";
        contentDiv.style.height = "100%";
        htmlTag.appendChild(contentDiv);

        var mapDiv = document.createElement("div");
        mapDiv.style.width = "100%";
        mapDiv.style.height = "100%";
        contentDiv.appendChild(mapDiv);

    $(self.htmlTag).dialog({
      title : self.getModel().getName(),
      width : Math.floor(window.innerWidth * 2 / 3),
      height : Math.floor(window.innerHeight * 2 / 3),
      position : {
        my : "center",
        at : "center",
        of : $(self.getTopMap().getElement()),
      },
      resize: function() {
        google.maps.event.trigger(self.getGoogleMap(), 'resize');
      }
    });

        $(self.htmlTag).dialog("open");

        self.setGoogleMap(new google.maps.Map(mapDiv, mapOptions));
        self._createMapChangedCallbacks();

    google.maps.event.trigger(self.getGoogleMap(), 'resize');

        if (self.isCustomTouchInterface()) {
            self._touchInterface = new TouchMap(self);
        }

        self.setupLayouts();

        self.registerMapClickEvents();

        var centerPoint = self.getModel().getCenterLatLng();
        self.getGoogleMap().setCenter(centerPoint);

        var sessionData = ServerConnector.getSessionData(this.getProject());
        // and now send the zoom level to the client side
        google.maps.event.addListener(self.getGoogleMap(), 'zoom_changed', function () {
            sessionData.setZoomLevel(self.getModel(), self.getGoogleMap().getZoom());
        });

        sessionData.setZoomLevel(self.getModel(), self.getGoogleMap().getZoom());

        self.initialized = true;
    } else {
        $(self.htmlTag).dialog("open");
    }

};

Submap.prototype.openDataOverlay = function (identifier) {
    if (this.initialized) {
        this.getGoogleMap().setMapTypeId(identifier.toString());
    }
};

Submap.prototype.getTopMap = function () {
    return this.getCustomMap();
};

Submap.prototype.getCustomMap = function () {
    return this._customMap;
};

Submap.prototype.setCustomMap = function (customMap) {
    this._customMap = customMap;
};

Submap.prototype.getProject = function () {
    return this.getCustomMap().getProject();
};

Submap.prototype.destroy = function () {
    var self = this;
    if (self.initialized) {
        $(self.htmlTag).dialog("destroy");
    }
};

module.exports = Submap;
