"use strict";

/* exported logger */

// noinspection JSUnusedLocalSymbols
var logger = require('../../logger');
var functions = require('../../Functions');

var AbstractSurfaceElement = require('./AbstractSurfaceElement');
var IdentifiedElement = require('../data/IdentifiedElement');

/**
 * Class representing overlay of the alias on the map relevant for a specific
 * layout.
 */
function AliasSurface(params) {
  // call super constructor
  AbstractSurfaceElement.call(this, params);

  // google map object associated with the alias
  this.addGoogleMapObject(params.gmapObj);

  // original data
  this.setBioEntity(params.alias);
  this.setIdentifiedElement(new IdentifiedElement(params.alias));
}

AliasSurface.prototype = Object.create(AbstractSurfaceElement.prototype);
AliasSurface.prototype.constructor = AliasSurface;

AliasSurface.prototype.setColor = function (color) {
  this._color = color;
  var googleMapObjects = this.getGoogleMapObjects();
  for (var i = 0; i < googleMapObjects.length; i++) {
    googleMapObjects[i].setOptions({
      strokeColor: color
    });
  }
};

/**
 * Function used to recalculate boundaries of the {@link AliasSurface}.
 * Boundaries define how big part of original alias is taken by this layout
 * visualization.
 *
 * @param startX
 *          value between 0..1 defining where should be the start on OX axis
 * @param endX
 *          value between 0..1 defining where should be the end on OX axis
 */
AliasSurface.prototype.setBoundsForAlias = function (startX, endX) {
  var alias = this.getBioEntity();
  var pointA = new google.maps.Point(alias.x + startX * alias.width, alias.y);
  var pointB = new google.maps.Point(alias.x + endX * alias.width, alias.y + alias.height);
  var latLngA = this.getCustomMap().fromPointToLatLng(pointA);
  var latLngB = this.getCustomMap().fromPointToLatLng(pointB);

  var bounds = new google.maps.LatLngBounds();
  bounds.extend(latLngA);
  bounds.extend(latLngB);
  this.getGoogleMapObjects()[0].setBounds(bounds);
};

/**
 * Creates {@link AliasSurface}.
 *
 * @param params
 *          dict containing set of information required for surface creation:
 *          <li>overlayAlias - {@link LayoutAlias} for which overlay is created
 *          <li>alias - {@link Alias data} of the alias
 *          <li>map - {@link AbstractCustomMap} where overlay should be placed
 *          <li> startX - this is the ratio on OX axis that should be use as a
 *          starting point of the overlay. For instance when there are three
 *          layouts to visualize then
 *          <ul>
 *          <li>the first layout have startX=0.0; endX=0.33333</li>
 *          <li>second layout have startX=0.33333; endX=0.66666</li>
 *          <li>the last layout have startX=0.66666; endX=1.0</li>
 *          </ul>
 *          <li>endX this is the ratio on OX axis that should be use as a
 *          starting point of the overlay
 * @returns {AliasSurface}
 */
AliasSurface.create = function (params) {
  var overlayAlias = params.overlayAlias;
  var alias = params.alias;
  var map = params.map;
  var startX = params.startX;
  var endX = params.endX;

  var pointA = new google.maps.Point(alias.x + startX * alias.width, alias.y);
  var pointB = new google.maps.Point(alias.x + endX * alias.width, alias.y + alias.height);
  var latLngA = map.fromPointToLatLng(pointA);
  var latLngB = map.fromPointToLatLng(pointB);

  var bounds = new google.maps.LatLngBounds();
  bounds.extend(latLngA);
  bounds.extend(latLngB);
  var rectangle = new google.maps.Rectangle({
    fillOpacity: 0.8,
    strokeWeight: 1,
    map: map.getGoogleMap(),
    bounds: bounds
  });

  return functions.overlayToColor(overlayAlias).then(function (color) {
    rectangle.setOptions({
      fillColor: color
    });
    return new AliasSurface({
      map: map,
      gmapObj: rectangle,
      alias: alias,
      onClick: params.onClick
    });
  });
};

AliasSurface.createFromIdentifiedElement = function (params) {
  var element = params.element;
  var map = params.map;
  var model = map.getModel().getSubmodelById(element.getModelId());
  return model.getByIdentifiedElement(element).then(function (alias) {
    var pointA = new google.maps.Point(alias.getX(), alias.getY());
    var pointB = new google.maps.Point(alias.getX() + alias.getWidth(), alias.getY() + alias.getHeight());
    var latLngA = map.fromPointToLatLng(pointA);
    var latLngB = map.fromPointToLatLng(pointB);

    var bounds = new google.maps.LatLngBounds();
    bounds.extend(latLngA);
    bounds.extend(latLngB);

    var color = "#FF0000";
    if (element.getColor() !== undefined) {
      color = element.getColor();
    }
    var fillOpacity = 0.8;
    if (element.getOpacity() !== undefined) {
      fillOpacity = element.getOpacity();
    }
    var strokeWeight = 1;
    if (element.getLineWeight() !== undefined) {
      strokeWeight = element.getLineWeight();
    }
    var strokeColor = "#000000";
    if (element.getLineColor() !== undefined) {
      strokeColor = element.getLineColor();
    }
    var strokeOpacity = 1;
    if (element.getLineOpacity() !== undefined) {
      strokeOpacity = element.getLineOpacity();
    }

    var rectangle = new google.maps.Rectangle({
      map: map.getGoogleMap(),
      bounds: bounds,

      fillOpacity: fillOpacity,
      fillColor: color,
      strokeColor: strokeColor,
      strokeOpacity: strokeOpacity,
      strokeWeight: strokeWeight
    });

    return new AliasSurface({
      gmapObj: rectangle,
      map: map,
      onClick: params.onClick,
      alias: alias
    });
  });
};

module.exports = AliasSurface;
